# -*- coding: utf-8 -*-
"""
@Author: ChenXiaolei
@Date: 2020-04-16 14:38:22
@LastEditTime: 2020-05-20 20:47:49
@LastEditors: ChenXiaolei
@Description: json 帮助类
"""
import json
import datetime
import decimal


class JsonEncoder(json.JSONEncoder):
    """
    继承json.JSONEncoder

    使用方法:json.dumps(json_obj, ensure_ascii=False, cls=JsonEncoder)
    """

    def default(self, obj):  # pylint: disable=E0202
        if isinstance(obj, datetime.datetime):
            return obj.strftime('%Y-%m-%d %H:%M:%S')
        elif isinstance(obj, datetime.date):
            return obj.strftime("%Y-%m-%d")
        elif isinstance(obj, bytes):
            return obj.decode()
        elif isinstance(obj, decimal.Decimal):
            return str(decimal.Decimal(obj).quantize(decimal.Decimal('0.00')))
        else:
            return json.JSONEncoder.default(self, obj)



# -*- coding: utf-8 -*-
"""
@Author: ChenXiaolei
@Date: 2020-05-19 10:33:52
@LastEditTime: 2020-06-02 10:36:35
@LastEditors: ChenXiaolei
@Description: ufile传输
"""
from ufile.filemanager import *
from ufile import multipartuploadufile
from io import BytesIO
# UCloud UFile


class UFileHelper(FileManager):

    def __init__(self, public_key, private_key, bucket=None, connection_timeout=300, upload_suffix=None, download_suffix=None, expires=None, user_agent=None, md5=None, cdn_prefix=None, src_prefix=None):
        """
        @description: 初始化 PutUFile 实例
        @param public_key: string类型, 账户API公私钥中的公钥
        @param private_key: string类型, 账户API公私钥中的私钥
        @param bucket: ufile空间名称
        @param connection_timeout: integer类型，网络请求超时时间
        @param upload_suffix: string类型，上传地址后缀
        @param download_suffix: string类型，下载地址后缀
        @param expires: integer类型，文件下载链接失效时间
        @param user_agent: string类型 user_agent
        @md5: 布尔类型，上传文件是否携带MD5
        @return: None，如果为非法的公私钥，则抛出ValueError异常
        """
        super(UFileHelper, self).__init__(public_key, private_key)
        self.public_key = public_key
        self.private_key = private_key
        self.bucket = bucket
        if cdn_prefix:
            self.cdn_prefix = cdn_prefix
        if src_prefix:
            self.src_prefix = src_prefix

        import ufile.config
        ufile.config.set_default(connection_timeout=connection_timeout, expires=expires,
                                 user_agent=user_agent, uploadsuffix=upload_suffix, downloadsuffix=download_suffix, md5=md5)

    def _get_bucket(self, bucket=None):
        """
        @description: 获取ufile bucket
        @param bucket: ufile空间名称
        @return: bucket
        @last_editors: ChenXiaolei
        """
        if bucket and bucket != "":
            return bucket
        elif hasattr(self, "bucket") and self.bucket != "":
            return self.bucket
        else:
            raise Exception("ufile bucket is not configured")

    def put_file(self, put_key, localfile, header=None, bucket=None):
        """
        @description: 上传文件至ufile
        @param put_key: string 类型，上传文件在空间中的名称
        @param localfile: string类型，本地文件名称
        @param header: dict类型，http 请求header，键值对类型分别为string，比如{'User-Agent': 'Google Chrome'}
        @param bucket: string类型，上传空间名称 初始化参数和此函数参数二选一传递
        @return: 字典类型,包含源文件=>src_url和cdn文件=>cdn_url
        @last_editors: ChenXiaolei
        """
        ret, resp = self.putfile(self._get_bucket(
            bucket), put_key, localfile, header=header)

        result = {}

        if resp.status_code == 200:
            if hasattr(self, "src_prefix") and self.src_prefix != "":
                result["src_url"] = self.src_prefix.rstrip('/')+"/"+put_key
            else:
                result["src_url"] = "/"+put_key

            if hasattr(self, "cdn_prefix") and self.cdn_prefix != "":
                result["cdn_url"] = self.cdn_prefix.rstrip('/')+"/"+put_key
            else:
                result["cdn_url"] = "/"+put_key

        return result

    def download_file(self, key, localfile, isprivate=False, content_range=None, header=None, bucket=None):
        """
        下载UFile文件并且保存为本地文件

        @param key: string类型， 下载文件在空间中的名称
        @param localfile: string类型，要保存的本地文件名称
        @param isprivate: boolean类型，如果为私有空间则为True
        @param content_range: tuple类型，元素为两个整型
        @param header: dict类型，http 请求header，键值对类型分别为string，比如{'User-Agent': 'Google Chrome'}
        @param bucket: string类型, UFile空间名称
        @return: ret: 如果http状态码为[200, 204, 206]之一则返回None，否则如果服务器返回json信息则返回dict类型，键值对类型分别为string, unicode string类型，否则返回空的dict
        @return: ResponseInfo: 响应的具体信息，UCloud UFile 服务器返回信息或者网络链接异常
        """
        bucket = self._get_bucket(bucket)

        return super().download_file(bucket, key, localfile, isprivate=isprivate,  content_range=content_range, header=header)



# -*- coding: utf-8 -*-
"""
@Author: ChenXiaolei
@Date: 2020-03-06 23:17:54
@LastEditTime: 2020-05-13 20:01:53
@LastEditors: ChenXiaolei
@Description: Handler基础类
"""

# seven_framework import
from .base_handler import *

# third package import
from pycket.session import SessionManager


class BaseCookieHandler(BaseHandler):
    """
    @description: api base handler. have session
    @last_editors: ChenXiaolei
    """

    def __init__(self, *argc, **argkw):
        """
        @description: 初始化
        @last_editors: ChenXiaolei
        """
        super(BaseCookieHandler, self).__init__(*argc, **argkw)
        self.session = SessionManager(self)

    def prepare_ext(self):
        """
        @description: 置于任何请求方法前被调用扩展
        @last_editors: ChenXiaolei
        """
        pass

    # 根据key 获取Session 内容
    def get_session_value(self, key):
        """
        @description: 二次封装获取session，无则返回默认值
        @param key: session key
        @return: session value
        @last_editors: ChenXiaolei
        """
        if key in self.session:
            return self.session[key]
        else:
            return None



# -*- coding: utf-8 -*-
"""
@Author: ChenXiaolei
@Date: 2020-04-16 14:38:22
@LastEditTime: 2020-05-06 20:30:57
@LastEditors: ChenXiaolei
@Description: 
"""
import base64
from Crypto.Cipher import AES


class CryptoHelper:
    """
    @description: 加密帮助类
    """
    @classmethod
    def md5_encrypt(self, source, salt="", encoding="utf-8"):
        """
        @description: md5加密，支持加盐算法
        @param source: 需加密的字符串
        @param salt: 加盐算法参数值
        @param encoding: 字符串编码
        @return: md5加密后的字符串
        @last_editors: ChenXiaolei
        """
        if not isinstance(source, str) or not isinstance(salt, str):
            return ""
        return self.md5_encrypt_bytes((source + salt).encode(encoding))

    @classmethod
    def md5_encrypt_bytes(self, source):
        """
        @description: md5字节流加密
        @param source: 字节流
        @return: md5加密后的字符串
        @last_editors: ChenXiaolei
        """
        if not isinstance(source, bytes):
            return ""
        import hashlib
        encrypt = hashlib.md5()
        encrypt.update(source)
        md5value = encrypt.hexdigest()
        return md5value

    @classmethod
    def md5_encrypt_int(self, source, salt="", encoding="utf-8"):
        """
        @description: md5加密，返回数值
        @param source: 需加密的字符串
        @param salt: 加盐算法参数值
        @return: md5加密后的数值
        @last_editors: ChenXiaolei
        """
        md5_16 = self._convert_md5(self.md5_encrypt(source, salt, encoding))
        hash_code_start = int.from_bytes(md5_16[0:8],
                                         byteorder='little',
                                         signed=True)
        hash_code_end = int.from_bytes(md5_16[8:16],
                                       byteorder='little',
                                       signed=True)
        return hash_code_start ^ hash_code_end

    @classmethod
    def _convert_md5(self, origin):
        """
        @description: md5字符串转16进制数组
        @param origin: 原md5字符串
        @return: 16进制数组
        @last_editors: ChenXiaolei
        """
        result = []
        s = ""
        for i in range(len(origin)):
            s += origin[i]
            if i % 2 != 0:
                int_hex = int(s, 16)
                result.append(int_hex)
                s = ""

        return result

    @classmethod
    def sha1_encrypt(self, source, encoding="utf-8"):
        """
        @description: sha1加密
        @param source: 需加密的字符串
        @return: sha1加密后的字符串
        @last_editors: ChenXiaolei
        """
        from hashlib import sha1
        sha1_sign = sha1()
        if isinstance(source, str):
            sha1_sign.update(source.encode(encoding))
        elif isinstance(source, bytes):
            sha1_sign.update(source)
        return sha1_sign.hexdigest()

    @classmethod
    def base64_encode(self, source, encoding="utf-8"):
        """
        @description: base64加密
        @param source: 需加密的字符串
        @return: 加密后的字符串
        @last_editors: ChenXiaolei
        """
        if not source.strip():
            return ""
        import base64
        encode_string = base64.b64encode(source.encode(encoding=encoding))
        return encode_string

    @classmethod
    def base64_decode(self, source):
        """
        @description: base64解密
        @param source: 需加密的字符串
        @return: 解密后的字符串
        @last_editors: ChenXiaolei
        """
        if not source.strip():
            return ""
        import base64
        decode_string = base64.b64decode(source)
        return decode_string

    @classmethod
    def _aes_pad(self, text, encoding="utf-8"):
        """
        @description: 填充函数，使被加密数据的字节码长度是block_size的整数倍
        @param text: 填充字符串
        @return: 填充后的字符串
        @last_editors: ChenXiaolei
        """
        length = AES.block_size
        count = len(text.encode(encoding))
        add = length - (count % length)
        entext = text + (chr(add) * add)
        return entext

    @classmethod
    def aes_encrypt(self, source, password, encoding="utf-8"):
        """
        @description: AES加密,ECB & PKCS7
        @param source: 待加密字符串
        @param password: 密钥 必须为16位或32位
        @return: 加密后的字符串
        @last_editors: ChenXiaolei
        """
        if isinstance(password, str):
            password = password.encode(encoding)

        aes = AES.new(password, AES.MODE_ECB)  # 初始化AES,ECB模式的实例

        resource = aes.encrypt(self._aes_pad(
            source, encoding).encode(encoding))
        result = str(base64.b64encode(resource), encoding=encoding)
        return result

    @classmethod
    def aes_decrypt(self, source, password, encoding="utf-8"):
        """
        @description: AES解密,ECB & PKCS7
        @param source: 待解密字符串
        @param password: 密钥 必须为16位或32位
        @return: 解密后的明文
        @last_editors: ChenXiaolei
        """
        if isinstance(password, str):
            password = password.encode(encoding)

        aes = AES.new(password, AES.MODE_ECB)  # 初始化AES,ECB模式的实例

        # 截断函数，去除填充的字符
        def unpad(date): return date[0:-ord(date[-1])]

        resource = base64.decodebytes(source.encode(encoding))
        result = aes.decrypt(resource).decode(encoding)
        return unpad(result)



# -*- coding: utf-8 -*-
"""
@Author: ChenXiaolei
@Date: 2020-04-16 21:32:43
@LastEditTime: 2020-05-15 20:27:22
@LastEditors: ChenXiaolei
@Description: 
"""
import requests


class HTTPHelper:
    """Constructs and sends a :class:`Request <Request>`.

    :param method: method for the new :class:`Request` object: ``GET``, ``OPTIONS``, ``HEAD``, ``POST``, ``PUT``, ``PATCH``, or ``DELETE``.
    :param url: URL for the new :class:`Request` object.
    :param params: (optional) Dictionary, list of tuples or bytes to send
        in the query string for the :class:`Request`.
    :param data: (optional) Dictionary, list of tuples, bytes, or file-like
        object to send in the body of the :class:`Request`.
    :param json: (optional) A JSON serializable Python object to send in the body of the :class:`Request`.
    :param headers: (optional) Dictionary of HTTP Headers to send with the :class:`Request`.
    :param cookies: (optional) Dict or CookieJar object to send with the :class:`Request`.
    :param files: (optional) Dictionary of ``'name': file-like-objects`` (or ``{'name': file-tuple}``) for multipart encoding upload.
        ``file-tuple`` can be a 2-tuple ``('filename', fileobj)``, 3-tuple ``('filename', fileobj, 'content_type')``
        or a 4-tuple ``('filename', fileobj, 'content_type', custom_headers)``, where ``'content-type'`` is a string
        defining the content type of the given file and ``custom_headers`` a dict-like object containing additional headers
        to add for the file.
    :param auth: (optional) Auth tuple to enable Basic/Digest/Custom HTTP Auth.
    :param timeout: (optional) How many seconds to wait for the server to send data
        before giving up, as a float, or a :ref:`(connect timeout, read
        timeout) <timeouts>` tuple.
    :type timeout: float or tuple
    :param allow_redirects: (optional) Boolean. Enable/disable GET/OPTIONS/POST/PUT/PATCH/DELETE/HEAD redirection. Defaults to ``True``.
    :type allow_redirects: bool
    :param proxies: (optional) Dictionary mapping protocol to the URL of the proxy.
    :param verify: (optional) Either a boolean, in which case it controls whether we verify
            the server's TLS certificate, or a string, in which case it must be a path
            to a CA bundle to use. Defaults to ``True``.
    :param stream: (optional) if ``False``, the response content will be immediately downloaded.
    :param cert: (optional) if String, path to ssl client cert file (.pem). If Tuple, ('cert', 'key') pair.
    :return: :class:`Response <Response>` object
    :rtype: requests.Response

    Usage::

      >>> import requests
      >>> req = requests.request('GET', 'https://httpbin.org/get')
      >>> req
      <Response [200]>
    """
    @classmethod
    def get(self, url, params=None, headers=None, **kwargs):
        """
        @description: http get请求
        @param url: 网络http地址
        @param param: 请求参数(字典)
        @param headers: 请求headers设置(字典)
        @return: 请求结果
        @last_editors: ChenXiaolei
        """
        try:
            return requests.get(url=url,
                                params=params,
                                headers=headers,
                                **kwargs)
        except:
            print(f"get请求url:{url},params:{params}异常")
            return None

    @classmethod
    def post(self,
             url,
             data=None,
             headers={'Content-Type': 'application/x-www-form-urlencoded'},
             **kwargs):
        """
        @description: http post请求
        @param url: 网络http地址
        @param data: 请求参数(字典)
        @param headers: 请求headers设置(字典)
        @return: 请求结果
        @last_editors: ChenXiaolei
        """
        try:
            if hasattr(headers, "Content-Type") and headers["headers"].find("json") > 0:
                return requests.post(url=url, json=data, headers=headers, **kwargs)
            return requests.post(url=url, data=data, headers=headers, **kwargs)
        except:
            print(f"get请求url:{url},params:{data}异常")
            return None



# -*- coding: utf-8 -*-
"""
@Author: ChenCheng
@Date: 2020-05-21 11:20:06
@LastEditTime: 2020-06-02 20:24:00
@LastEditors: ChenXiaolei
@Description: excel help
"""


import openpyxl
import os
import sys
import re
import xlwt
import xlrd
from xlutils.copy import copy

from seven_framework import *


class ExcelHelper(object):
    """
    @description: 对excel基本操作进行封装的类
    @last_editors: ChenCheng
    """

    @classmethod
    def __get_filename_by_xlsx(self):
        return os.path.abspath('.') + r'\新建Excel.xlsx'

    @classmethod
    def __error_file_type(self):
        raise TypeError  # "文件类型错误"

    @classmethod
    def __get_filename_by_xls(self):
        return os.path.abspath('.') + r'\新建Excel.xls'

    @classmethod
    def judge_format(self, filename):
        """
        @description: 判断文件格式
        @param filename: 文件名
        @return: 文件格式 'xls' or 'xlsx'
        @last_editors: ChenCheng
        """
        file_type = re.search('xls[x]*', filename)
        if file_type:
            if file_type.group() == 'xls':
                return 'xls'
            else:
                return 'xlsx'
        else:
            self.__error_file_type()

    @classmethod
    def create_file(self, filename=''):
        """
        @description: 创建excel文件
        @param filename: 文件名（包含完整路径）,默认在当前文件夹下创建一个"新建Excel.xlsx"
        @return: 
        @last_editors: ChenCheng
        """
        if not filename:
            filename = self.__get_filename_by_xlsx()

        file_type = self.judge_format(filename)
        if file_type == 'xls':
            self._create_file_by_xls(filename)
        elif file_type == 'xlsx':
            self._create_file_by_xlsx(filename)
        else:
            self.__error_file_type()

    @classmethod
    def input(self, filename, sheet_name='Sheet'):
        """
        @description: 从excel导入数据
        @param filename：文件名，需要完整路径
        @return data:二维数组形式输出数组
        @last_editors: ChenCheng
        """
        cell_num_begin = [1, 1]
        max_row = self.get_max_row(filename, sheet_name=sheet_name)
        max_column = self.get_max_column(filename, sheet_name=sheet_name)
        cell_num_end = [max_row, max_column]
        data = self.read_cell(
            cell_num_begin, cell_num_end, filename, sheet_name)
        return data

    @classmethod
    def export(self, data, filename=''):
        """
        @description: 数据导出到excel
        @param data: 字典数组 
        @param filename: 文件名（包含地址）,默认在当前文件夹下创建一个"新建Excel.xlsx" 
        @return: 
        @last_editors: ChenCheng
        """
        if not filename:
            filename = self.__get_filename_by_xlsx()
        else:
            if self.judge_format(filename) == 'xls':
                f = filename + 'x'
                filename = f
        print(f'开始：{filename}')
        self._create_file_by_xlsx(filename)

        # 生成二维数组格式data
        columns = 0
        row_first = dict()

        for dict_data in data:
            for item in dict_data:
                if item not in row_first:
                    columns += 1
                    row_first[item] = columns

        # 数组初始化
        list_list_data = [['' for j in range(columns)]
                          for i in range(len(data) + 1)]
        for item, value in row_first.items():
            list_list_data[0][value - 1] = item

        column = 1
        for dict_data in data:
            column += 1
            for item, value in dict_data.items():
                list_list_data[column - 1][row_first[item] - 1] = value

        cell_num_begin = [1, 1]
        self.updata_cells(list_list_data, cell_num_begin, filename)

    @classmethod
    def updata_cell(self, row, column, value, filename='', sheet_name='Sheet'):
        """
        @description: 单个单元格更新数据
        @param row, column：单元格坐标（row, column）
        @param value：要更新的数据
        @param filename: 文件名（包含地址）,默认在当前文件夹下创建一个"新建Excel.xlsx" 
        @param sheet_name: 表单名
        @return: 
        @last_editors: ChenCheng
        """
        data = [[value, ], ]
        cell_num_begin = [row, column]

        self.updata_cells(data, cell_num_begin, filename, sheet_name)

    @classmethod
    def updata_cells(self, data, cell_num_begin, filename='', sheet_name='Sheet'):
        """
        @description: 更新单元格数据
        @param data: 需要更新的数据, 二维数组
        @param cell_num_begin: 单元格起始位置编号,如[1,1]，注意：第一格坐标为[1,1]
        @param filename: 文件名（包含地址）,默认在当前文件夹下创建一个"新建Excel.xlsx" 
        @param sheet_name: 表单名
        @return: 
        @last_editors: ChenCheng
        """
        if not filename:
            self._create_file_by_xlsx(filename)
            filename = self.__get_filename_by_xlsx()

        file_type = self.judge_format(filename)
        if file_type == 'xls':
            if sheet_name == 'Sheet':
                sheet_name = 'Sheet1'
            self._updata_cell_by_xls(
                data, cell_num_begin, filename, sheet_name)
        elif file_type == 'xlsx':
            self._updata_cell_by_xlsx(
                data, cell_num_begin, filename, sheet_name)
        else:
            self.__error_file_type()

    @classmethod
    def read_cell(self, cell_num_begin, cell_num_end, filename='', sheet_name='Sheet'):
        """
        @description: 读取指定位置单元格数据
        @param cell_num_begin: 单元格起始位置编号,如[1,1]，注意：第一格坐标为[1,1]
        @param cell_num_end: 单元格j结束位置编号,如[2,2]
        @param filename: 文件名（包含地址）,默认在当前文件夹下创建一个"新建Excel.xls" 
        @param sheet_name: 表单名
        @return data:返回输入范围内的数据，二维数组，格式为data[[],[],...]
        @last_editors: ChenCheng
        """
        if not filename:
            self._create_file_by_xls(filename)
            filename = self.__get_filename_by_xlsx()

        file_type = self.judge_format(filename)
        if file_type == 'xls':
            if sheet_name == 'Sheet':
                sheet_name = 'Sheet1'
            return self._read_cell_by_xls(cell_num_begin, cell_num_end, filename, sheet_name)
        elif file_type == 'xlsx':
            return self._read_cell_by_xlsx(cell_num_begin, cell_num_end, filename, sheet_name)
        else:
            self.__error_file_type()

    @classmethod
    def delete_cell(self, cell_num_begin, cell_num_end, filename, sheet_name='Sheet'):
        """
        @description: 读取指定位置单元格数据
        @param cell_num_begin: 单元格起始位置编号,如[1,1]，注意：第一格坐标为[1,1]
        @param cell_num_end: 单元格j结束位置编号,如[2,2]
        @param filename: 文件名（包含地址）
        @param sheet_name: 表单名
        @return:
        @last_editors: ChenCheng
        """
        file_type = self.judge_format(filename)
        if file_type == 'xls':
            if sheet_name == 'Sheet':
                sheet_name = 'Sheet1'
            self._delete_cell_by_xls(
                cell_num_begin, cell_num_end, filename, sheet_name)
        elif file_type == 'xlsx':
            self._delete_cell_by_xlsx(
                cell_num_begin, cell_num_end, filename, sheet_name)
        else:
            self.__error_file_type()

    @classmethod
    def get_max_row(self, filename, sheet_name='Sheet'):
        """
        @description: 获取最大行数
        @param filename: 文件名
        @param sheet_name: 表单名
        @return: 返回选定表单有数据的最大行数
        @last_editors: ChenCheng
        """
        if self.judge_format(filename) == 'xls':
            if sheet_name == 'Sheet':
                sheet_name = 'Sheet1'
            excel = xlrd.open_workbook(filename)
            sheet = excel.sheet_by_name(sheet_name)
            return sheet.nrows
        else:
            excel = openpyxl.load_workbook(filename)
            sheet = excel[sheet_name]
            return sheet.max_row

    @classmethod
    def get_max_column(self, filename, sheet_name='Sheet'):
        """
        @description: 获取最大列数
        @param filename: 文件名
        @param sheet_name: 表单名
        @return: 返回选定表单有数据的最大列数
        @last_editors: ChenCheng
        """
        if self.judge_format(filename) == 'xls':
            if sheet_name == 'Sheet':
                sheet_name = 'Sheet1'
            excel = xlrd.open_workbook(filename)
            sheet = excel.sheet_by_name(sheet_name)
            return sheet.ncols
        else:
            excel = openpyxl.load_workbook(filename)
            sheet = excel[sheet_name]
            return sheet.max_column

    @classmethod
    def create_sheet(self, sheet_name, filename=''):
        """
        @description: 创建表单，只支持xlsx
        @param sheet_name: 表单名字
        @param filename: 文件名（包含地址）,默认在当前文件夹下创建一个"新建Excel.xlsx"
        @return: 
        @last_editors: ChenCheng
        """
        if not filename:
            self.create_file_by_xls(filename)
            filename = self.__get_filename_by_xls()

        if self.judge_format(filename) == 'xlsx':
            excel = openpyxl.load_workbook(filename)
            excel.createe_sheet(sheet_name)
            excel.save(filename)
        else:
            print('只支持xlsx创建表单')
            self.__error_file_type()

    @classmethod
    def delete_sheet(self, sheet_name, filename):
        """
        @description: 删除表单，只支持xlsx
        @param sheet_name: 表单名字
        @param filename: 文件名
        @return: 
        @last_editors: ChenCheng
        """
        if self.judge_format(filename) == 'xlsx':
            excel = openpyxl.load_workbook(filename)
            sheet = excel[sheet_name]
            excel.remove(sheet)
            excel.save(filename)
        else:
            print('只支持xlsx删除表单')
            self.__error_file_type()

    @classmethod
    def _create_file_by_xls(self, filename=''):
        """
        @description: 创建xls文件
        @param filename: 文件名（包含地址）,默认在当前文件夹下创建一个"新建Excel.xls"
        @return: 
        @last_editors: ChenCheng
        """
        if not filename:
            filename = self.__get_filename_by_xls()

        if self.judge_format(filename) == 'xls':
            excel = xlwt.Workbook()
            sheet = excel.add_sheet('Sheet1')
            excel.save(filename)
        else:
            self.__error_file_type()

    @classmethod
    def _create_file_by_xlsx(self, filename=''):
        """
        @description: 创建xlsx文件
        @param filename: 文件名（包含地址）,默认在当前文件夹下创建一个"新建Excel.xlsx"
        @return: 
        @last_editors: ChenCheng
        """
        if not filename:
            filename = self.__get_filename_by_xlsx()

        if self.judge_format(filename) == 'xlsx':
            excel = openpyxl.Workbook()
            sheet = excel.active
            excel.save(filename)
        else:
            self.__error_file_type()

    @classmethod
    def _updata_cell_by_xls(self, data, cell_num_begin,  filename='', sheet_id=0):
        """
        @description: 更新单元格数据
        @param data: 需要更新的数据, 二维数组
        @param cell_num_begin: 单元格起始位置编号,如[1,1]，注意：第一格坐标为[1,1]
        @param filename: 文件名（包含地址）,默认在当前文件夹下创建一个"新建Excel.xlsx" 
        @param sheet_id: 表单id
        @return: 
        @last_editors: ChenCheng
        """
        if not filename:
            self._create_file_by_xls(filename)
            filename = self.__get_filename_by_xls()

        if self.judge_format(filename) == 'xls':
            excel = xlrd.open_workbook(filename)
            excel_copy = copy(excel)
            #sheet = excel_copy.sheet_by_index(sheet_name)
            # updata_cell（）的默认值传入，修改默认值
            if sheet_id == 'Sheet1':
                sheet_id = 0
            sheet = excel_copy.get_sheet(sheet_id)
            # 导入数据
            for i in range(cell_num_begin[0], cell_num_begin[0] + len(data)):
                for j in range(cell_num_begin[1], cell_num_begin[1] + len(data[0])):
                    sheet.write(
                        i-1, j-1, data[i - cell_num_begin[0]][j - cell_num_begin[1]])
            # 保存
            excel_copy.save(filename)
        else:
            self.__error_file_type()

    @classmethod
    def _updata_cell_by_xlsx(self, data, cell_num_begin,  filename='', sheet_name='Sheet'):
        """
        @description: 更新单元格数据
        @param data: 需要更新的数据, 二维数组
        @param cell_num_begin: 单元格起始位置编号,如[1,1]，注意：第一格坐标为[1,1]
        @param filename: 文件名（包含地址）,默认在当前文件夹下创建一个"新建Excel.xlsx" 
        @param sheet_name: 表单名
        @return: 
        @last_editors: ChenCheng
        """
        if not filename:
            self._create_file_by_xls(filename)
            filename = self.__get_filename_by_xlsx()

        if self.judge_format(filename) == 'xlsx':
            excel = openpyxl.load_workbook(filename)
            sheet = excel[sheet_name]
            # 导入数据
            for i in range(cell_num_begin[0], cell_num_begin[0] + len(data)):
                for j in range(cell_num_begin[1], cell_num_begin[1] + len(data[0])):
                    cell = openpyxl.utils.get_column_letter(j) + str(i)
                    sheet[cell] = data[i - cell_num_begin[0]
                                       ][j - cell_num_begin[1]]
            # 保存
            excel.save(filename)
        else:
            self.__error_file_type()

    @classmethod
    def _read_cell_by_xls(self, cell_num_begin, cell_num_end, filename='', sheet_name='Sheet1'):
        """
        @description: 读取指定位置单元格数据
        @param cell_num_begin: 单元格起始位置编号,如[1,1]，注意：第一格坐标为[1,1]
        @param cell_num_end: 单元格j结束位置编号,如[2,2]
        @param filename: 文件名（包含地址）,默认在当前文件夹下创建一个"新建Excel.xls" 
        @param sheet_name: 表单名
        @return data:返回输入范围内的数据，格式为data[[],[],...]
        @last_editors: ChenCheng
        """
        if not filename:
            self._create_file_by_xls(filename)
            filename = self.__get_filename_by_xls()

        if self.judge_format(filename) == 'xls':
            excel = xlrd.open_workbook(filename)
            sheet = excel.sheet_by_name(sheet_name)
            # 读取数据
            data = []
            for i in range(cell_num_begin[0], cell_num_end[0] + 1):
                data_row = []
                for j in range(cell_num_begin[1], cell_num_end[1] + 1):
                    data_row.append(sheet.cell(i-1, j-1).value)
                data.append(data_row)
            return data
        else:
            self.__error_file_type()

    @classmethod
    def _read_cell_by_xlsx(self, cell_num_begin, cell_num_end, filename='', sheet_name='Sheet'):
        """
        @description: 读取指定位置单元格数据
        @param cell_num_begin: 单元格起始位置编号,如[1,1]，注意：第一格坐标为[1,1]
        @param cell_num_end: 单元格j结束位置编号,如[2,2]
        @param filename: 文件名（包含地址）,默认在当前文件夹下创建一个"新建Excel.xlsx" 
        @param sheet_name: 表单名
        @return data:返回输入范围内的数据，二维数组，格式为data[[],[],...]
        @last_editors: ChenCheng
        """
        if not filename:
            self._create_file_by_xlsx(filename)
            filename = self.__get_filename_by_xlsx()

        if self.judge_format(filename) == 'xlsx':
            excel = openpyxl.load_workbook(filename)
            sheet = excel[sheet_name]
            # 读取数据
            data = []
            for i in range(cell_num_begin[0], cell_num_end[0] + 1):
                data_row = []
                for j in range(cell_num_begin[1], cell_num_end[1] + 1):
                    cell = openpyxl.utils.get_column_letter(j) + str(i)
                    data_row.append(sheet[cell].value)
                data.append(data_row)
            return data
        else:
            self.__error_file_type()

    @classmethod
    def _delete_cell_by_xls(self, cell_num_begin, cell_num_end, filename, sheet_name='Sheet1'):
        """
        @description: 读取指定位置单元格数据
        @param cell_num_begin: 单元格起始位置编号,如[1,1]，注意：第一格坐标为[1,1]
        @param cell_num_end: 单元格j结束位置编号,如[2,2]
        @param filename: 文件名（包含地址）
        @param sheet_name: 表单名
        @return :
        @last_editors: ChenCheng
        """
        if not filename:
            self._create_file_by_xlsx(filename)

        if self.judge_format(filename) == 'xls':
            data = []
            for i in range(cell_num_begin[0], cell_num_end[0] + 1):
                data_row = []
                for j in range(cell_num_begin[1], cell_num_end[1] + 1):
                    data_row.append('')
                data.append(data_row)
            self._updata_cell_by_xls(
                data, cell_num_begin, filename, sheet_name)
        else:
            self.__error_file_type()

    @classmethod
    def _delete_cell_by_xlsx(self, cell_num_begin, cell_num_end, filename, sheet_name='Sheet'):
        """
        @description: 读取指定位置单元格数据
        @param cell_num_begin: 单元格起始位置编号,如[1,1]，注意：第一格坐标为[1,1]
        @param cell_num_end: 单元格j结束位置编号,如[2,2]
        @param filename: 文件名（包含地址）
        @param sheet_name: 表单名
        @return:
        @last_editors: ChenCheng
        """
        if self.judge_format(filename) == 'xlsx':
            excel = openpyxl.load_workbook(filename)
            sheet = excel[sheet_name]
            # 删除数据
            for i in range(cell_num_begin[0], cell_num_end[0] + 1):
                for j in range(cell_num_begin[1], cell_num_end[1] + 1):
                    cell = openpyxl.utils.get_column_letter(j) + str(i)
                    sheet[cell] = ''
            excel.save(filename)
        else:
            self.__error_file_type()



# -*- coding: utf-8 -*-
"""
@Author: ChenXiaolei
@Date: 2020-03-06 23:17:54
@LastEditTime: 2020-06-18 19:14:36
@LastEditors: ChenXiaolei
@Description: 数据库基础操作类
"""

from seven_framework.mysql import MySQLHelper
import datetime
import os
from seven_framework import *


class BaseModel:
    def __init__(self, model_class, sub_table):
        """
        @description: 基础数据库操作类
        @param model_class: 实体对象类
        @param sub_table: 分表标识
        @last_editors: ChenXiaolei
        """
        # 实体对象类
        self.model_class = model_class
        # 实体对象
        self.model_obj = model_class()
        # 数据库表名
        self.table_name = str(self.model_obj) if not sub_table else str(
            self.model_obj).lower().replace("_tb", f"_{sub_table}_tb")
        # 主键字段名
        self.primary_key_field = self.model_obj.get_primary_key()

    def __convert_field_type(self, field_value):
        """
        @description: 数据库字段类型兼容转换
        @param field_value: 字段值
        @return: 转换类型后的字段值
        @last_editors: ChenXiaolei
        """
        if isinstance(field_value, datetime.datetime):
            return field_value.strftime('%Y-%m-%d %H:%M:%S')
        elif isinstance(field_value, datetime.date):
            return field_value.strftime("%Y-%m-%d")
        elif isinstance(field_value, bytes):
            return field_value.decode()
        elif isinstance(field_value, decimal.Decimal):
            return str(decimal.Decimal(field_value).quantize(decimal.Decimal('0.00')))
        return field_value

    def __row_entity(self, data):
        """
        @description: 单条数据转成对象
        @param data :数据字典
        @return: 模型实体
        @last_editors: ChenXiaolei
        """
        field_list = self.model_obj.get_field_list()
        if data is None or len(field_list) == 0:
            return None
        model_entity = self.model_class()
        for field_str in field_list:
            data[field_str] = self.__convert_field_type(data[field_str])
            setattr(model_entity, field_str, data[field_str])
        return model_entity

    def __row_entity_list(self, data_list):
        """
        @description: 数据列表返回对象
        @param data_list: 数据字典数组
        @return: 模型实体列表
        @last_editors: ChenXiaolei
        """
        field_list = self.model_obj.get_field_list()
        if data_list is None or len(field_list) == 0:
            return None
        model_entity_list = []
        if len(data_list) > 0:
            for data in data_list:
                model_entity = self.model_class()
                for field_str in field_list:
                    if field_str in data:
                        data[field_str] = self.__convert_field_type(
                            data[field_str])
                        setattr(model_entity, field_str, data[field_str])
                model_entity_list.append(model_entity)
        return model_entity_list

    def get_list(self, where='', group_by='', order_by='', limit='', params=None):
        """
        @description: 根据条件获取列表
        @param where: 数据库查询条件语句
        @param group_by: GROUP BY 语句
        @param order_by:  ORDER BY 语句
        @param limit:  LIMIT 语句
        @param params: 参数化查询参数
        @return: 模型实体列表
        @last_editors: ChenXiaolei
        """
        if where and where.strip() != '':
            where = " WHERE " + where
        if group_by and group_by.strip() != '':
            group_by = " GROUP BY "+group_by
        if order_by and order_by.strip() != '':
            order_by = " ORDER BY " + order_by
        if limit and limit.strip() != '':
            limit = " LIMIT "+limit
        sql = f"SELECT * FROM {self.table_name}{where}{group_by}{order_by}{limit};"
        list_row = self.db.fetch_all_rows(sql, params)
        return self.__row_entity_list(list_row)

    def get_page_list(self,
                      field,
                      page_index,
                      page_size,
                      where='',
                      group_by='',
                      order_by='',
                      params=None):
        """
        @description: 分页获取数据
        @param field: 查询字段 
        @param page_index: 分页页码 0为第一页
        @param page_size: 分页返回数据数量
        @param where: 数据库查询条件语句
        @param group_by: GROUP BY 语句
        @param order_by:  ORDER BY 语句
        @param params: 参数化查询参数
        @return: 模型实体列表
        @last_editors: ChenXiaolei
        """
        if where and where.strip() != '':
            where = " WHERE " + where
        if group_by and group_by.strip() != '':
            group_by = " GROUP BY " + group_by
        if order_by and order_by.strip() != '':
            order_by = " ORDER BY " + order_by

        sql = f"SELECT {field} FROM {self.table_name}{where}{group_by}{order_by} LIMIT {str(int(page_index) * int(page_size))},{str(page_size)};"
        list_row = self.db.fetch_all_rows(sql, params)

        sql = f"SELECT COUNT({self.primary_key_field}) AS count FROM {self.table_name}{where}{group_by};"
        row = self.db.fetch_one_row(sql, params)

        if row and 'count' in row and int(row['count']) > 0:
            row_count = int(row["count"])
        else:
            row_count = 0

        return self.__row_entity_list(list_row), row_count

    def get_entity(self, where='', group_by='', order_by='',  params=None):
        """
        @description: 根据条件获取实体对象
        @param where: 数据库查询条件语句
        @param group_by: GROUP BY 语句
        @param order_by:  ORDER BY 语句
        @param params: 参数化查询参数
        @return: 模型实体
        @last_editors: ChenXiaolei
        """
        if where and where.strip() != '':
            where = " WHERE " + where
        if group_by and group_by.strip() != '':
            group_by = " GROUP BY "+group_by
        if order_by and order_by.strip() != '':
            order_by = " ORDER BY " + order_by
        sql = f"SELECT * FROM {self.table_name}{where}{group_by}{order_by} LIMIT 1;"
        list_row = self.db.fetch_one_row(sql, params)
        return self.__row_entity(list_row)

    def get_entity_by_id(self, primary_key_id):
        """
        @description: 根据主键值获取实体对象
        @param primary_key_id: 主键ID值 
        @return: 模型实体
        @last_editors: ChenXiaolei
        """
        sql = f"SELECT * FROM {self.table_name} WHERE {self.primary_key_field}=%s;"
        list_row = self.db.fetch_one_row(sql, primary_key_id)
        return self.__row_entity(list_row)

    def get_total(self, where='', group_by='', field=None, params=None):
        """
        @description: 根据条件获取数据数量
        @param where: 数据库查询条件语句
        @param group_by: GROUP BY 语句
        @param params: 参数化查询参数
        @param field: count(传参)
        @return: 查询符合条件的行的数量
        @last_editors: ChenXiaolei
        """
        if where and where.strip() != '':
            where = " WHERE " + where
        if group_by and group_by.strip() != '':
            group_by = " GROUP BY "+group_by
        if not field:
            field = self.primary_key_field
        sql = f"SELECT COUNT({field}) AS count FROM {self.table_name}{where}{group_by};"
        list_row = self.db.fetch_one_row(sql, params)
        return list_row['count']

    def del_entity(self, where, params=None):
        """
        @description: 根据条件删除数据库中的数据
        @param where: 数据库查询条件语句
        @param params: 参数化查询参数
        @return: 
        @last_editors: ChenXiaolei
        """
        if where:
            where = " WHERE " + where
            sql = "DELETE FROM " + self.table_name + where

            if self.is_transaction():
                transaction_item = {}
                transaction_item["sql"] = sql
                transaction_item["params"] = params
                self.db_transaction.transaction_list.append(transaction_item)
                return True
            else:
                list_row = self.db.delete(sql, params)
                if list_row is not None and list_row >= 0:
                    return True
                else:
                    return False

    def add_entity(self, model, ignore=False):
        """
        @description: 数据入库
        @param model: 模型实体 
        @return: 如果主键为自增ID，返回主键值
        @last_editors: ChenXiaolei
        """
        field_list = self.model_obj.get_field_list()
        if len(field_list) == 0:
            return 0
        insert_field_str = ""
        insert_value_str = ""
        param = []
        for field_str in field_list:
            if str(field_str).lower() == self.primary_key_field:
                continue
            insert_field_str += str(field_str + ",")
            insert_value_str += "%s,"
            param.append(str(getattr(model, field_str)))
        insert_field_str = insert_field_str.rstrip(',')
        insert_value_str = insert_value_str.rstrip(',')
        sql = f"INSERT{' IGNORE' if ignore else ''} INTO {self.table_name}({insert_field_str}) VALUE({insert_value_str});"
        if self.is_transaction():
            transaction_item = {}
            transaction_item["sql"] = sql
            transaction_item["params"] = tuple(param)
            self.db_transaction.transaction_list.append(transaction_item)
            return True
        else:
            list_row = self.db.insert(sql, tuple(param))
            return list_row

    def add_list(self, model_list, ignore=False, transaction=False):
        """
        @description: 实体列表入库
        @param model_list: 实体列表
        @param ignore: 忽略已存在的记录
        @param transaction: 是否事务提交
        @return: 执行成功True 执行失败False
        @last_editors: ChenXiaolei
        """
        if not model_list or len(model_list) == 0:
            return False
        sql_list = []
        field_list = self.model_obj.get_field_list()
        if len(field_list) == 0:
            return False
        for model in model_list:
            insert_field_str = ""
            insert_value_str = ""
            param = []
            for field_str in field_list:
                if str(field_str).lower() == self.primary_key_field:
                    continue
                insert_field_str += str(field_str + ",")
                insert_value_str += "%s,"
                param.append(str(getattr(model, field_str)))
            insert_field_str = insert_field_str.rstrip(',')
            insert_value_str = insert_value_str.rstrip(',')
            sql = f"INSERT{' IGNORE' if ignore else ''} INTO {self.table_name}({insert_field_str}) VALUE({insert_value_str});"
            if self.is_transaction():
                transaction_item = {}
                transaction_item["sql"] = sql
                transaction_item["params"] = tuple(param)
                self.db_transaction.transaction_list.append(transaction_item)
            elif transaction:
                sql_list.append(sql)
            else:
                self.db.insert(sql, tuple(param))
        if transaction and not self.is_transaction():
            if not self.transaction_execute(sql_list):
                return False
        return True

    def add_update_entity(self, model, update_sql, params=None):
        """
        @description: 数据入库,遇到主键冲突则更新指定字段
        @param model: 模型实体 
        @param update_sql: 如果主键冲突则执行的更新sql语句
        @param params: 参数化查询参数
        @return: 如果主键为自增ID，返回主键值
        @last_editors: ChenXiaolei
        """
        field_list = self.model_obj.get_field_list()
        if len(field_list) == 0:
            return 0
        insert_field_str = ""
        insert_value_str = ""
        param = []
        for field_str in field_list:
            if str(field_str).lower() == self.primary_key_field:
                continue
            insert_field_str += str(field_str + ",")
            insert_value_str += "%s,"
            param.append(str(getattr(model, field_str)))
        if params:
            param = param+params
        insert_field_str = insert_field_str.rstrip(',')
        insert_value_str = insert_value_str.rstrip(',')
        sql = f"INSERT INTO {self.table_name}({insert_field_str}) VALUE({insert_value_str}) ON DUPLICATE KEY UPDATE {update_sql};"
        if self.is_transaction():
            transaction_item = {}
            transaction_item["sql"] = sql
            transaction_item["params"] = tuple(param)
            self.db_transaction.transaction_list.append(transaction_item)
            return True
        else:
            list_row = self.db.insert(sql, tuple(param))
            return list_row

    def update_entity(self, model, field_list=None):
        """
        @description: 根据模型的主键ID，更新字段的值
        @param model: 模型实体
        @return: 更新成功返回主键ID，否则返回空字符串
        @last_editors: ChenXiaolei
        """
        field_list = self.model_obj.get_field_list()
        if len(field_list) == 0:
            return 0
        update_field_str = ""

        param = []
        mid = ""
        for field_str in field_list:
            if self.primary_key_field and str(field_str).lower() == self.primary_key_field.lower():
                mid = getattr(model, field_str)
                continue
            update_field_str += str(field_str + "=%s,")
            if str(field_str).lower() == "edit_on":
                now = datetime.datetime.now()
                param.append(str(now.strftime("%Y-%m-%d %H:%M:%S")))
            else:
                param.append(str(getattr(model, field_str)))
        param.append(mid)
        update_field_str = update_field_str.rstrip(',')
        if mid == 0:
            return 0
        sql = f"UPDATE {self.table_name} SET {update_field_str} WHERE {self.primary_key_field}=%s;"
        if self.is_transaction():
            transaction_item = {}
            transaction_item["sql"] = sql
            transaction_item["params"] = tuple(param)
            self.db_transaction.transaction_list.append(transaction_item)
            return True
        else:
            data = self.db.update(sql, tuple(param))
            if data is not None and data >= 0:
                return mid
            else:
                return ""

    def update_table(self, update_sql, where, params=None):
        """
        @description: 更新数据表
        @param update_sql: 更新set语句
        @param where: 数据库查询条件语句
        @param params: 参数化查询参数
        @return: 更新成功即为True 失败则为False
        @last_editors: ChenXiaolei
        """
        if where and where.strip() != '':
            where = " WHERE " + where
        sql = f"UPDATE {self.table_name} SET {update_sql}{where};"
        if self.is_transaction():
            transaction_item = {}
            transaction_item["sql"] = sql
            transaction_item["params"] = params
            self.db_transaction.transaction_list.append(transaction_item)
            return True
        else:
            data = self.db.update(sql, params)
            if data is not None and data >= 0:
                return True
            else:
                return False

    def get_dict(self, where='', group_by='', order_by='', limit='', field="*", params=None):
        """
        @description: 返回字典dict
        @param where: 数据库查询条件语句
        @param group_by: GROUP BY 语句
        @param order_by:  ORDER BY 语句
        @param limit:  LIMIT 语句
        @param field: 查询字段 
        @param params: 参数化查询参数
        @return: 返回匹配条件的第一行字典数据
        @last_editors: ChenXiaolei
        """
        if where and where.strip() != '':
            where = " WHERE " + where
        if group_by and group_by.strip() != '':
            group_by = " GROUP BY "+group_by
        if order_by and order_by.strip() != '':
            order_by = " ORDER BY " + order_by
        if limit and limit.strip() != '':
            limit = " LIMIT "+limit
        sql = f"SELECT {field} FROM {self.table_name}{where}{group_by}{order_by}{limit}"
        one_row = self.db.fetch_one_row(sql, params)
        return one_row  # self.__row_entity(list_row)

    def get_dict_by_id(self, primary_key_id, field="*"):
        """
        @description: 根据主键ID获取dict
        @param primary_key_id: 主键id值 
        @param field: 查询字段 
        @return: 返回匹配id的第一行字典数据
        @last_editors: ChenXiaolei
        """
        sql = f"SELECT {field} FROM {self.table_name} WHERE {self.primary_key_field}=%s;"
        one_row = self.db.fetch_one_row(sql, primary_key_id)
        return one_row

    def get_dict_list(self, where='', group_by='', order_by='', limit='', field="*", params=None):
        """
        @description: 返回字典列表dict list
        @param where: 数据库查询条件语句
        @param group_by: GROUP BY 语句
        @param order_by:  ORDER BY 语句
        @param limit:  LIMIT 语句
        @param field: 查询字段 
        @param params: 参数化查询参数
        @return: 
        @last_editors: ChenXiaolei
        """
        if where and where.strip() != '':
            where = " WHERE " + where
        if group_by and group_by.strip() != '':
            group_by = " GROUP BY "+group_by
        if order_by and order_by.strip() != '':
            order_by = " ORDER BY " + order_by
        if limit and limit.strip() != '':
            limit = " LIMIT "+limit
        sql = f"SELECT {field} FROM {self.table_name}{where}{group_by}{order_by}{limit};"
        list_row = self.db.fetch_all_rows(sql, params)
        return list_row

    def get_dict_page_list(self,
                           field,
                           page_index,
                           page_size,
                           where='',
                           group_by='',
                           order_by='',
                           params=None):
        """
        @description: 获取分页字典数据
        @param field: 查询字段 
        @param page_index: 分页页码 0为第一页
        @param page_size: 分页返回数据数量
        @param where: 数据库查询条件语句
        @param group_by: GROUP BY 语句
        @param order_by:  ORDER BY 语句
        @param params: 参数化查询参数
        @return: 数据字典数组
        @last_editors: ChenXiaolei
        """
        if where and where.strip() != '':
            where = " WHERE " + where
        if group_by and group_by.strip() != '':
            group_by = " GROUP BY " + group_by
        if order_by and order_by.strip() != '':
            order_by = " ORDER BY " + order_by

        sql = f"SELECT {field} FROM {self.table_name}{where}{group_by}{order_by} LIMIT {str(int(page_index) * int(page_size))},{str(page_size)}"
        list_row = self.db.fetch_all_rows(sql, params)

        sql = f"SELECT COUNT({self.primary_key_field}) AS count FROM {self.table_name}{where}{group_by};"
        row = self.db.fetch_one_row(sql, params)

        if row and 'count' in row and int(row['count']) > 0:
            row_count = int(row["count"])
        else:
            row_count = 0

        return list_row, row_count

    def transaction_execute(self, sql_list):
        """
        @description: 执行事务,失败回滚
        @param sql_list:事务SQL字符串数组
        @return: 执行成功True 执行失败False
        @last_editors: ChenXiaolei
        """
        if not sql_list or len(sql_list) == 0:
            return False
        return self.db.transaction_execute(sql_list)

    def is_transaction(self):
        """
        @description: 是否开启事务
        @last_editors: ChenXiaolei
        """
        if hasattr(self, "db_transaction") and self.db_transaction:
            return True
        return False



# -*- coding: utf-8 -*-
"""
@Author: ChenXiaolei
@Date: 2020-04-16 21:32:43
@LastEditTime: 2020-06-01 15:46:43
@LastEditors: ChenXiaolei
@Description: 
"""

import logging
import logging.handlers
import time
import os
import json
import socket
import platform

from .mysql import *


class Logger:
    """
    指定保存日志的文件路径，日志级别，以及调用文件 将日志存入到指定的文件中
    级别优先级:NOTSET < DEBUG < INFO < WARNING < ERROR < CRITICAL
    """

    def __init__(self, log_file_name, log_level, logger, host_ip="", project_name=None, db_connect=None, db_table_name="python_log_tb"):
        """
        @description: 
        @param log_file_name: 日志存储文件路径
        @param log_level: 日志等级
        @param logger: 日志标识
        @param host_ip: 服务器IP
        @param project_name: 项目标志
        @last_editors: ChenXiaolei
        """
        if not project_name:
            if platform.system() == "Windows":
                project_name = os.getcwd().split('\\')[-1]
            else:
                project_name = os.getcwd().split('/')[-1]

        # 判断文件夹是否存在，不存在则创建
        path_list = log_file_name.split("/")
        path_log = log_file_name[0:log_file_name.find(path_list[len(path_list)
                                                                - 1])]
        if not os.path.isdir(path_log):
            os.mkdir(path_log)

        logging_level = ''

        if log_level.upper() == 'NOTSET':
            logging_level = logging.NOTSET
        elif log_level.upper() == 'DEBUG':
            logging_level = logging.DEBUG
        elif log_level.upper() == 'INFO':
            logging_level = logging.INFO
        elif log_level.upper() == 'WARNING':
            logging_level = logging.WARNING
        elif log_level.upper() == 'ERROR':
            logging_level = logging.ERROR
        elif log_level.upper() == 'CRITICAL':
            logging_level = logging.CRITICAL

        # 创建一个logger
        self.logger = logging.getLogger(logger)
        self.logger.setLevel(logging_level)

        # 创建handler，用于写入日志文件
        self.handler_file = logging.handlers.TimedRotatingFileHandler(
            log_file_name, 'D', 1, 10)
        # 设置 切分后日志文件名的时间格式 默认 log_file_name+"." + suffix 如果需要更改需要改logging 源码
        self.handler_file.suffix = "%Y%m%d.log"
        self.handler_file.setLevel(logging_level)

        formatter = logging.Formatter(
            json.dumps({
                "record_time": "%(asctime)s",
                "level": "%(levelname)s",
                "log_msg": "%(message)s",
                "host_ip": host_ip,
                "project_name": project_name
            }))
        self.handler_file.setFormatter(formatter)

        # 创建handler，用于输出至控制台
        # 定义控制台输出handler的输出格式
        formatter = logging.Formatter(
            '[%(asctime)s][%(name)s][%(levelname)s]%(message)s')
        self.handler_console = logging.StreamHandler()  # 输出到控制台的handler
        self.handler_console.setFormatter(formatter)

        # 给logger添加handler
        if not self.logger.handlers:
            self.logger.addHandler(self.handler_file)
            self.logger.addHandler(self.handler_console)

        if db_connect:
            self.handler_db = LoggerMysqlHandler(
                db_connect, db_table_name, project_name, host_ip)
            self.logger.addHandler(self.handler_db)

    def close(self):
        self.logger.removeHandler(self.handler_file)
        self.handler_file.close()

        if hasattr(self, "handler_console"):
            self.logger.removeHandler(self.handler_console)
            self.handler_console.close()
        if hasattr(self, "handler_db"):
            self.logger.removeHandler(self.handler_db)
            self.handler_db.close()

    def get_logger(self):
        return self.logger

    @classmethod
    def get_logger_by_name(self, loger_name):
        """
        @description: 通过日志标识获取logger
        @param loger_name: 日志标识
        @return: logger
        @last_editors: ChenXiaolei
        """
        return logging.getLogger(loger_name)


class LoggerMysqlHandler(logging.Handler):
    def __init__(self, db_connect, table_name, project_name="", host_ip=""):
        try:
            if not db_connect:
                raise "未配置日志传输数据库配置"
                return
            self.db = MySQLHelper(db_connect)
            self.table_name = table_name
            self.project_name = project_name
            self.host_ip = host_ip
            # 判断制定数据库中标是否存在
            exist_result = self.db.fetch_one_row(
                f"SELECT TABLE_NAME FROM INFORMATION_SCHEMA.TABLES WHERE TABLE_SCHEMA='{db_connect['db']}' AND TABLE_NAME='{table_name}';")

            if not exist_result:
                self.db.fetch_and_commit(
                    f"CREATE TABLE `{table_name}` ( `id` int(11) NOT NULL AUTO_INCREMENT COMMENT '主键id', `project_name` varchar(50) DEFAULT NULL COMMENT '项目名称', `level` varchar(20) DEFAULT NULL COMMENT '日志等级', `host_ip` varchar(50) DEFAULT NULL COMMENT '服务器ip', `record_time` datetime NOT NULL COMMENT '记录时间', `record_timestamp` int(11) NOT NULL COMMENT '记录时间',`log_msg` longtext NOT NULL COMMENT '日志内容', PRIMARY KEY (`id`) ) ENGINE=InnoDB DEFAULT CHARSET=utf8;")
            logging.Handler.__init__(self)
        except Exception as ex:
            print(ex)
            pass

    def emit(self, record):
        try:
            self.db.fetch_and_commit(
                f"INSERT INTO {self.table_name}(project_name,level,host_ip,record_time,record_timestamp,log_msg) VALUE('{self.project_name}','{record.levelname}','{self.host_ip}','{record.asctime}',{int(record.created)},'{pymysql.escape_string(record.message)}');")
        except Exception as ex:
            print(ex)
            pass



# -*- coding: utf-8 -*-
"""
@Author: ChenXiaolei
@Date: 2020-04-16 14:38:22
@LastEditTime: 2020-06-02 20:27:52
@LastEditors: ChenXiaolei
@Description: 
"""

from . import config
from .coding import *
from .crypto import *
from .mysql import *
from .log import *
from .time import *
from .uuid import *
from .http import * 
from .json import *
from .host import *
from .notice import *
from .excel import *
from .qr_code import *
# from .ufile import *
# from . import web_tornado
# from . import console


# -*- coding: utf-8 -*-
"""
@Author: ChenXiaolei
@Date: 2020-04-25 21:58:58
@LastEditTime: 2020-04-28 09:46:50
@LastEditors: ChenXiaolei
@Description: 
"""

# from . import base_api_handler
# from . import base_cookie_handler
# from . import base_handler


'''
@Author: ChenXiaolei
@Date: 2020-03-14 16:12:44
@LastEditTime: 2020-03-19 15:37:32
@LastEditors: ChenXiaolei
@Description: mysql helper
@FilePath: /seven_manage_api/libs./seven_framework/mysql.py
'''
# -*- coding: utf-8 -*-
import pymysql
from DBUtils.PooledDB import PooledDB
import traceback


class MySQLHelper:
    # 对MySQLdb常用函数进行封装的类

    error_code = ''  # MySQL错误号码
    _instance = None  # 本类的实例
    conn = None  # 数据库conn
    _cur = None

    def __init__(self, dbconfig):
        """
        @description: mysql 操作类初始化
        @param dbconfig: 连接字符串
        @last_editors: ChenXiaolei
        """
        self.POOL = PooledDB(
            creator=pymysql,  # 使用链接数据库的模块
            maxconnections=0,  # 连接池允许的最大连接数，0和None表示不限制连接数
            mincached=0,  # 初始化时，链接池中至少创建的空闲的链接，0表示不创建
            maxcached=0,  # 链接池中最多闲置的链接，0和None不限制
            maxshared=0,  # 链接池中最多共享的链接数量，0和None表示全部共享。PS: 无用，因为pymysql和MySQLdb等模块的 threadsafety都为1，所有值无论设置为多少，_maxcached永远为0，所以永远是所有链接都共享。
            blocking=True,  # 连接池中如果没有可用连接后，是否阻塞等待。True，等待；False，不等待然后报错
            maxusage=None,  # 一个链接最多被重复使用的次数，None表示无限制
            setsession=[],  # 开始会话前执行的命令列表。如：["set datestyle to ...", "set time zone ..."]
            ping=0,
            # ping MySQL服务端，检查是否服务可用。# 如：0 = None = never, 1 = default = whenever it is requested, 2 = when a cursor is created, 4 = when a query is executed, 7 = always
            host=dbconfig['host'],
            port=dbconfig['port'],
            user=dbconfig['user'],
            passwd=dbconfig['passwd'],
            db=dbconfig['db'],
            charset=dbconfig['charset'])

    def query(self, sql, params=None):
        """
        @description: 执行 SELECT 语句
        @param sql: 查询语句
        @param params: 参数值
        @return: 查询结果
        @last_editors: ChenXiaolei
        """
        # 执行 SELECT 语句
        self._execute(sql, params)
        record_list = self._cur.fetchall()
        self.close()
        return record_list

    def update(self, sql, params=None):
        """
        @description: 执行 UPDATE 操作
        @param sql: 查询语句
        @param params: 参数值
        @return: 执行结果
        @last_editors: ChenXiaolei
        """
        self._execute(sql, params)
        self._conn.commit()
        row_count = self._cur.rowcount
        self.close()
        return row_count

    def delete(self, sql, params=None):
        """
        @description: 执行 Delete 操作
        @param sql: 查询语句
        @param params: 参数值
        @return: 执行结果
        @last_editors: ChenXiaolei
        """
        self._execute(sql, params)
        self._conn.commit()
        row_count = self._cur.rowcount
        self.close()
        return row_count

    def insert(self, sql, params=None):
        """
        @description: 执行 INSERT 语句
        @param sql: 数据insert语句
        @return: 如主键为自增长int，则返回新生成的ID
        @last_editors: ChenXiaolei
        """
        # 执行 INSERT 语句。如主键为自增长int，则返回新生成的ID
        self._execute(sql, params)
        self._conn.commit()
        primary_id = self._cur._result.insert_id
        self.close()
        return primary_id

    def _execute(self, sql, params=None):
        self.connection()

        try:
            self._cur = self._conn.cursor(pymysql.cursors.DictCursor)
            if not params:
                self._cur.execute(sql)
            else:
                self._cur.execute(sql, params)
            return self._cur
        except Exception as e:
            print(traceback.print_exc())
            raise Exception("ERROR:" + str(e) + " SQL：" +
                            str(sql) + " 参数：" + str(params))

    def transaction_execute(self, sql_list):
        """
        @description: 事务提交
        @param sql_list: sql字符串数组
        @return: 执行成功返回 True   失败 False
        @last_editors: ChenXiaolei
        """
        try:
            for item in sql_list:
                if type(item) == str:
                    self._execute(item)
                elif type(item) == dict:
                    sql = item["sql"]
                    params = item["params"]
                    self._execute(sql, params)
        except Exception as e:
            self.rollback()  # 事务回滚
            # raise Exception("执行事务ERROR:" + str(e) + " SQL：" + str(sql))
            return False
        else:
            self.commit()  # 事务提交
        self.close()
        return True

    def fetch_all_rows(self, sql, params=None):
        """
        @description: 返回结果列表
        @param sql: 查询语句
        @param params: 参数值
        @return: 结果列表
        @last_editors: ChenXiaolei
        """
        self._execute(sql, params)
        record_list = self._cur.fetchall()
        self.close()
        return record_list

    def fetch_limit_rows(self, sql, params=None, current_page=1, list_rows=20):
        """
        @description: 分页查询结果
        @param sql: 查询语句
        @param params: 参数值
        @return: 分页查询结果
        @last_editors: ChenXiaolei
        """
        count = self.get_row_count(sql, params)
        pages = count / list_rows
        pages = pages + 1 if not count % list_rows == 0 else pages
        if (pages == 0):
            pages = 1
        if (current_page < 1):
            current_page = 1
        if (current_page > pages):
            current_page = pages
        start = (current_page - 1) * list_rows
        end = list_rows
        previous_page = current_page - 1 if current_page > 1 else 1
        next_page = current_page + \
            1 if current_page < int(pages) else int(pages)
        limit_sql = sql + " limit " + str(int(start)) + "," + str(int(end))
        result = {}
        result["list"] = self.fetch_all_rows(limit_sql, params)
        result["page"] = {
            "prev": previous_page,
            "next": next_page,
            "current": current_page,
            "pages": int(pages),
            "total": count,
        }

        return result

    def fetch_one_row(self, sql, params=None):
        """
        @description: 返回一行结果，然后游标指向下一行。到达最后一行以后，返回None
        @param sql: 查询语句
        @param params: 参数值
        @return: 返回一行结果
        @last_editors: ChenXiaolei
        """
        self._execute(sql, params)
        record = self._cur.fetchone()
        self.close()
        return record

    def get_row_count(self, sql, params=None):
        """
        @description: 获取结果行数
        @param sql: 查询语句
        @param params: 参数值
        @return: 返回查询行数
        @last_editors: ChenXiaolei
        """
        self._execute(sql, params)
        row_count = self._cur.rowcount
        self.close()
        return row_count

    def fetch_and_commit(self, sql, params=None):
        """
        @description: 执行并commit提交
        @param sql: 查询语句
        @param params: 参数值
        @return: 返回第一行结果
        @last_editors: ChenXiaolei
        """
        self._execute(sql, params)
        record = self._cur.fetchone()
        self._conn.commit()
        self.close()
        return record

    def commit(self):
        """
        @description: 数据库commit操作
        @last_editors: ChenXiaolei
        """
        # 数据库commit操作
        self._conn.commit()

    def rollback(self):
        """
        @description: 执行数据库回滚操作
        @last_editors: ChenXiaolei
        """
        # 数据库回滚操作'
        self._conn.rollback()

    def connection(self):
        self._conn = self.POOL.connection()

    def __del__(self):
        """
        @description: 执行释放资源（系统GC自动调用）
        @last_editors: ChenXiaolei
        """
        # 释放资源（系统GC自动调用）
        try:
            self._cur.close()
            self._conn.close()
        except:
            pass

    def close(self):
        """
        @description: 执行关闭数据库连接
        @last_editors: ChenXiaolei
        """
        # 关闭数据库连接
        self.__del__()


class DbTransaction():
    """
    @description: 数据库事务执行类
    @last_editors: ChenXiaolei
    """
    def __init__(self,db=None,db_config_dict=None):
         self.transaction_list = []

         if db:
             self.db=db
         elif db_config_dict and type(db_config_dict)==dict:
             self.db = MySQLHelper(db_config_dict)
         else:
             raise Exception("db not configured!")

    def commit_transaction(self):
        """
        @description: 提交事务
        @return: 成功True  失败False
        @last_editors: ChenXiaolei
        """
        if not hasattr(self,"transaction_list"):
            return False
        
        sql_list = self.transaction_list

        delattr(self, "transaction_list")

        return self.db.transaction_execute(sql_list)

    def rollback_transaction(self):
        """
        @description: 撤销事务
        @return: 成功True  失败False
        @last_editors: ChenXiaolei
        """
        delattr(self, "transaction_list")
        return True



# -*- coding: utf-8 -*-
"""
@Author: YuMinJie
@Date: 2020-05-15 11:16:28
@LastEditTime: 2020-06-10 14:04:27
@LastEditors: ChenXiaolei
@Description: 消息帮助类
"""


import requests
import platform
import os


class NoticeHelper(object):

    """
    @description: 企业微信群机器人消息
    """

    def __init__(self, webhook_key='', project_name=None):

        if webhook_key and webhook_key != '':
            # 参数获取
            # 传入webhook秘钥
            self.webhook_key = webhook_key

        if not project_name:
            if platform.system() == "Windows":
                self.project_name = os.getcwd().split('\\')[-1]
            else:
                self.project_name = os.getcwd().split('/')[-1]
        else:
            self.project_name = project_name

    def _get_webhook_key(self, webhook_key=''):
        """
        @description: 获取webhook_key，判断传参webhook_key是否为空字符串，如果为空字符串则从初始化属性获取
        @param webhook_key: webhook密钥，默认值为空字符串
        @return: webhook_key

        """
        if not webhook_key or webhook_key == '':
            if not hasattr(self, "webhook_key") or self.webhook_key == '':
                raise Exception("webhook_key is not configured")
            return self.webhook_key
        return webhook_key

    def send_webhook(self, text='', mentioned_list=[], mentioned_mobile_list=[], webhook_key=''):
        """
        @description: 发送企业微信消息
        @param webhook_key: webhook秘钥
        @param text:消息文本
        @param mentioned_list:工号、企业微信名或者@all
        @param mentioned_mobile_list:传入手机号或者@all
        @return: 企业微信消息

        """

        webhook_key = self._get_webhook_key(webhook_key)
        # 数据合成

        url = "https://qyapi.weixin.qq.com/cgi-bin/webhook/send?key=" + webhook_key
        body = {"msgtype": "text", "text": {"content": f"【{self.project_name}】{text}",
                                            "mentioned_list": mentioned_list, 'mentioned_mobile_list': mentioned_mobile_list}}

        try:
            response = requests.post(url, json=body, auth=(
                "Content-Type", "application/json"))
        except Exception as ex:
            pass



# -*- coding: utf-8 -*-
"""
@Author: ChenXiaolei
@Date: 2020-05-11 21:46:17
@LastEditTime: 2020-05-25 16:29:57
@LastEditors: ChenXiaolei
@Description: 
"""

from urllib import parse


class CodingHelper():
    @classmethod
    def url_encode(self, text, coding='utf-8'):
        return parse.quote_plus(text, encoding='utf-8')

    @classmethod
    def url_decode(self, text, coding='utf-8'):
        return parse.unquote(text, 'utf-8')



# -*- coding: utf-8 -*-
"""
@Author: ChenXiaolei
@Date: 2020-04-26 14:11:31
@LastEditTime: 2020-05-28 11:23:16
@LastEditors: ChenXiaolei
@Description: 
"""
import socket


class HostHelper:
    @classmethod
    def get_host_ip(self):
        """
        查询本机ip地址
        :return: ip
        """
        ip = ""
        try:
            s = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
            s.connect(('114.114.114.114', 80))
            ip = s.getsockname()[0]
        except:
            pass
        finally:
            s.close()

        return ip



# -*- coding: utf-8 -*-
"""
@Author: ChenXiaolei
@Date: 2020-05-09 20:39:20
@LastEditTime: 2020-05-25 15:28:33
@LastEditors: ChenXiaolei
@Description: 基础控制台类
"""

from seven_framework import *
import sys
global environment
if "--production" in sys.argv:
    environment = "production"
    config_file = "config.json"
elif "--testing" in sys.argv:
    environment = "testing"
    config_file = "config_testing.json"
else:
    environment = "development"
    config_file = "config_dev.json"

sys.path.append(".local")  # 不可删除,置于其他import前
# 初始化配置,执行顺序需先于调用模块导入
config.init_config(config_file)  # 全局配置,只需要配置一次

# 初始化日志写入类调用方法
logger_error = None
logger_info = None
log_config = config.get_value("log")

if log_config and type(log_config) == dict and "db_log" in log_config:
    db_log = log_config["db_log"]
    db_log_table_name = log_config["db_log_table_name"] if "db_log_table_name" in log_config else "python_log_tb"
    log_info_to_db = log_config["log_info_to_db"] if "log_info_to_db" in log_config else True
    log_error_to_db = log_config["log_error_to_db"] if "log_error_to_db" in log_config else True

    if log_error_to_db:
        logger_error = Logger("logs/log_error", "ERROR", "log_error",
                              HostHelper.get_host_ip(), db_connect=db_log, db_table_name=db_log_table_name).get_logger()
    if log_info_to_db:
        logger_info = Logger("logs/log_info", "INFO", "log_info",
                             HostHelper.get_host_ip(), db_connect=db_log, db_table_name=db_log_table_name).get_logger()

if not logger_error:
    logger_error = Logger("logs/log_error", "ERROR", "log_error",
                          HostHelper.get_host_ip()).get_logger()
if not logger_info:
    logger_info = Logger("logs/log_info", "INFO", "log_info",
                         HostHelper.get_host_ip()).get_logger()



# -*- coding: utf-8 -*-
"""
@Author: ChenXiaolei
@Date: 2020-06-02 20:25:11
@LastEditTime: 2020-06-03 19:41:14
@LastEditors: ChenXiaolei
@Description: QR Code Helper  
@Description: Doc:https://github.com/lincolnloop/python-qrcode#advanced-usage
"""

import io
import qrcode


class QRCodeHelper(object):
    """
    @description: 二维码帮助类
    """
    @classmethod
    def create_qr_code(self, data, version=1, error_correction=qrcode.constants.ERROR_CORRECT_M, box_size=10, border=4, fill_color="green", back_color="white"):
        """
        @description: 
        @param data: 二维码数据
        @param version: 二维码的格子矩阵大小，可以是1到40，1最小为21*21，40是177*177
        @param error_correction: 二维码错误容许率，默认ERROR_CORRECT_M，容许小于15%的错误率
        @param box_size: 二维码每个小格子包含的像素数量
        @param border: 二维码到图片边框的小格子数，默认值为4
        @param fill_color: 填充颜色 默认绿色
        @param back_color: 背景颜色 默认白色
        @return: 图片流
        @last_editors: HuangJingCan
        """
        # 实例化二维码生成类
        qr = qrcode.QRCode(
            version=version,
            error_correction=error_correction,
            box_size=box_size,
            border=border,
        )
        # 设置二维码数据
        qr.add_data(data=data)

        # 启用二维码颜色设置
        qr.make(fit=True)
        img = qr.make_image(fill_color=fill_color, back_color=back_color)

        # 显示二维码
        # img.show()

        img_byte_arr = io.BytesIO()
        img.save(img_byte_arr, format='JPEG')
        img_bytes = img_byte_arr.getvalue()

        return img, img_bytes



# -*- coding: utf-8 -*-
"""
@Author: ChenXiaolei
@Date: 2020-03-06 23:17:54
@LastEditTime: 2020-05-27 16:30:11
@LastEditors: ChenXiaolei
@Description: Handler基础类
"""

# seven_framework import
from .base_handler import *


class BaseApiHandler(BaseHandler):
    """
    @description: api base handler. not session
    @last_editors: ChenXiaolei
    """

    def __init__(self, *argc, **argkw):
        """
        @description: 初始化
        @last_editors: ChenXiaolei
        """
        super(BaseApiHandler, self).__init__(*argc, **argkw)

    def write_error(self, status_code, **kwargs):
        """
        @description: 重写全局异常事件捕捉
        @last_editors: ChenXiaolei
        """
        self.logger_error.error(traceback.format_exc())
        return self.reponse_json_error()

    def prepare_ext(self):
        """
        @description: 置于任何请求方法前被调用扩展
        @last_editors: ChenXiaolei
        """
        pass
